-- ObjcTest2
-- *Note: need to include "Cameras" dependencies to this project from Codea Craft menu in order to touch rotate 3D object


function setup()
    viewer.mode = FULLSCREEN
    -- Create a new 3D Codea Craft scene
    scene = craft.scene()
    -- change the main camera position by "camera" properaty of scene
    scene.camera.z = -4
    -- to actually change the camera's function (e.g. it's viewport) need to actually get the camera element of camera element
    scene.cameraElement = scene.camera:get(craft.camera)
    -- to change the scene camera's viewport (i.e. where 3D scene drawn on display) pass relative values x,y,width,height
    -- note: don't need to include the WIDTH/WIDTH or HEIGHT/HEIGHT expressions but written to illustrate using relative values
    scene.cameraElement:viewport((0.65*(WIDTH/WIDTH)),(0.15*(HEIGHT/HEIGHT)),(0.33*(WIDTH/WIDTH)),(0.5*(HEIGHT/HEIGHT)))
    
    -- Create a new 3D Codea entity to display in a 3D viewport portion of the screen
    Cube = scene:entity()
    Cube.model = craft.model(asset.builtin.Blocky_Characters.Robot)
    Cube.scale = vec3(0.3,0.3,0.3)
    Cube.eulerAngles = vec3(0,0,0) 
    scene.camera.z = -4
    angle = 0
    
    -- add OrbitViewer (come with Craft "Cameras" dependency attached)
    scene.camera:add(OrbitViewer, Cube.position, 10, 5, 20)
    
    --**Final part of setup below sets up ObjC code to work with Codea**
    
    --note: the next statement gets the actual UIViewController running in Codea
    local vc = objc.viewer
    
    --note: the next statement gets makes an instance (new) UI text view that you can use to make a GUI textbox
    uiTextView = objc.cls.UITextView()
    vc.view:addSubview_(uiTextView)
    
    --note: need to make the next ObjC statement false to allow use of advanced Apple ObjC display formatting constraints
    uiTextView.translatesAutoresizingMaskIntoConstraints = false
    
    -- Pin the trailing edge of the ObjC textbox (uiTextView) 20 in from right side of screen
    uiTextView.trailingAnchor:constraintEqualToAnchor_constant_(vc.view.trailingAnchor, -20).active = true
    
    -- Pin the top edge of the ObjC textbox (uiTextView) 20 down from top of screen)
    uiTextView.topAnchor:constraintEqualToAnchor_constant_(vc.view.topAnchor, 20).active = true
    
    -- Set the width and height of ObjC textbox (uiTextView)
    uiTextView.widthAnchor:constraintEqualToConstant_(400).active = true
    uiTextView.heightAnchor:constraintEqualToConstant_(200).active = true
    uiTextView.text = "Hello World."
    -- Apple's ObjC GUI objects like uiTextView allow lots of customization
    uiTextView.layer.cornerRadius = 8
    
    -- placeholder variable for what user types in the textbox
    typedText=""
    
    --create a delegate to work with the UITextView that will sense when changes are made to the textbox
    Delegate = objc.delegate("UITextViewDelegate")
    
    function Delegate:textViewShouldBeginEditing_(objTextView)
        -- replace with false to prevent editing textbox
        return true
    end
    
    function Delegate:textViewDidChange_(objTextView)
        typedText = objTextView.text
    end
    
    -- set delegate to textView AFTER you've created the delegate above
    uiTextView.delegate = Delegate()
    
    -- SFSafariViewController is an ObjC fully-functional self-contained browser that you can resize & include in Codea app
    local URL = objc.cls.NSURL
    local url = URL:URLWithString_("http://www.google.com")
    local SFSafariViewController = objc.cls.SFSafariViewController
    local safariBrowser = SFSafariViewController:alloc():initWithURL_(url)
    vc:addChildViewController_(safariBrowser)
    vc.view:addSubview_(safariBrowser.view)
    safariBrowser.view.translatesAutoresizingMaskIntoConstraints = false
    safariBrowser.view.leadingAnchor:constraintEqualToAnchor_constant_(vc.view.leadingAnchor, 20).active = true
    safariBrowser.view.topAnchor:constraintEqualToAnchor_constant_(vc.view.topAnchor, 20).active = true
    safariBrowser.view.widthAnchor:constraintEqualToConstant_(450).active = true
    safariBrowser.view.heightAnchor:constraintEqualToConstant_(700).active = true   
end

function update(dt)
    -- Update the Codea 3D Craft scene (physics, transforms etc)
    angle = angle + 1
    Cube.eulerAngles = vec3(angle/10, angle, angle/5)
    scene:update(dt)
end

function draw()
    background(0)
    update(DeltaTime)
    
    -- Actually draw a 3d Codea Craft scene
    scene:draw()
    
    -- Use standard Codea Graphics to write to the screen what is in the ObjC Textbox
    fill(160, 223, 157)
    textWrapWidth(260)
    text(typedText, WIDTH/2 + 20, HEIGHT-200)
end

